@@ -25,7 +25,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: XSECC14n20010315.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XSECC14n20010315.cpp 1493975 2013-06-17 23:43:09Z scantor $
  *
  */
 
@@ -39,6 +39,7 @@
 // Xerces includes
 #include <xercesc/dom/DOMElement.hpp>
 #include <xercesc/dom/DOMNamedNodeMap.hpp>
+#include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 
 XERCES_CPP_NAMESPACE_USE
@@ -240,6 +241,8 @@ void XSECC14n20010315::setExclusive(char * xmlnsList) {
 
 	}
 
+	ArrayJanitor<char> j_nsBuf(nsBuf);
+
 	int i, j;
 
 	i = 0;
@@ -247,21 +250,22 @@ void XSECC14n20010315::setExclusive(char * xmlnsList) {
 	while (xmlnsList[i] != '\0') {
 
 		while (xmlnsList[i] == ' ' ||
-			   xmlnsList[i] == '\0' ||
 			   xmlnsList[i] == '\t' ||
 			   xmlnsList[i] == '\r' ||
-			   xmlnsList[i] == '\n')
+			   xmlnsList[i] == '\n') {
 
 			   ++i;	// Skip white space
+		}
 
 		j = 0;
 		while (!(xmlnsList[i] == ' ' ||
 			   xmlnsList[i] == '\0' ||
 			   xmlnsList[i] == '\t' ||
 			   xmlnsList[i] == '\r' ||
-			   xmlnsList[i] == '\n'))
+			   xmlnsList[i] == '\n')) {
 
 			   nsBuf[j++] = xmlnsList[i++];	// Copy name
+		}
 
 		// Terminate the string
 		nsBuf[j] = '\0';
@@ -281,8 +285,6 @@ void XSECC14n20010315::setExclusive(char * xmlnsList) {
 
 	}
 
-	delete[] nsBuf;
-
 }
 
 
