@@ -24,7 +24,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: OpenSSLCryptoKeyDSA.cpp 1125752 2011-05-21 17:50:17Z scantor $
+ * $Id: OpenSSLCryptoKeyDSA.cpp 1655515 2015-01-29 03:11:28Z scantor $
  *
  */
 #include <xsec/framework/XSECDefs.hpp>
@@ -277,7 +277,7 @@ unsigned int OpenSSLCryptoKeyDSA::signBase64Signature(unsigned char * hashBuf,
 
 	// Now turn the signature into a base64 string
 
-	unsigned char* rawSigBuf = new unsigned char[(BN_num_bits(dsa_sig->r) + BN_num_bits(dsa_sig->s)) / 8];
+	unsigned char* rawSigBuf = new unsigned char[(BN_num_bits(dsa_sig->r) + BN_num_bits(dsa_sig->s) + 7) / 8];
     ArrayJanitor<unsigned char> j_sigbuf(rawSigBuf);
 	
     unsigned int rawLen = BN_bn2bin(dsa_sig->r, rawSigBuf);
