@@ -25,7 +25,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: OpenSSLCryptoSymmetricKey.cpp 1353308 2012-06-24 18:43:47Z scantor $
+ * $Id: OpenSSLCryptoSymmetricKey.cpp 1655510 2015-01-29 02:20:39Z scantor $
  *
  */
 
@@ -488,7 +488,7 @@ unsigned int OpenSSLCryptoSymmetricKey::decryptFinish(unsigned char * plainBuf,
 		// Should never see any bytes output, as we are not padding
 
 		throw XSECCryptoException(XSECCryptoException::SymmetricError,
-			"OpenSSL:SymmetricKey - Unexpectedly recieved bytes from EVP_DecryptFinal_ex"); 
+			"OpenSSL:SymmetricKey - Unexpectedly received bytes from EVP_DecryptFinal_ex");
 
 	}
 
