@@ -25,7 +25,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: OpenSSLCryptoSymmetricKey.hpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: OpenSSLCryptoSymmetricKey.hpp 1352680 2012-06-21 20:58:07Z scantor $
  *
  */
 
@@ -142,12 +142,16 @@ public :
 	 * @param mode mode selection (Currently ECB or CBC mode only)
 	 * @param iv Initialisation Vector to be used.  NULL if one is
 	 * not required, or if IV will be set from data stream
+     * @param tag Authentication tag to be used for AEAD ciphers
+     * @param taglen length of Authentication Tag
 	 * @returns true if the initialisation succeeded.
 	 */
 
 	virtual bool decryptInit(bool doPad = true,
 							 SymmetricKeyMode mode = MODE_CBC,
-							 const unsigned char * iv = NULL);
+							 const unsigned char * iv = NULL,
+                             const unsigned char* tag = NULL,
+                             unsigned int taglen = NULL);
 
 	/**
 	 * \brief Continue an decrypt operation using this key.
@@ -262,11 +266,13 @@ public :
 	 *
 	 * @param plainBuf Buffer to place final block of cipher text in
 	 * @param maxOutLength Maximum number of bytes to pace in output
+     * @param taglen length of Authentication Tag
 	 * @returns Bytes placed in output buffer
 	 */
 
 	virtual unsigned int encryptFinish(unsigned char * plainBuf,
-									   unsigned int maxOutLength);
+									   unsigned int maxOutLength,
+                                       unsigned int taglen = 0);
 
 	//@}
 
@@ -296,13 +302,14 @@ private:
 	OpenSSLCryptoSymmetricKey & operator= (const OpenSSLCryptoSymmetricKey &);
 
 	// Private functions
-	int decryptCtxInit(const unsigned char * iv);
+	int decryptCtxInit(const unsigned char* iv, const unsigned char* tag, unsigned int taglen);
 
 	// Private variables
 	SymmetricKeyType				m_keyType;
 	SymmetricKeyMode				m_keyMode;
 	EVP_CIPHER_CTX					m_ctx;			// OpenSSL Cipher Context structure
 	safeBuffer						m_keyBuf;		// Holder of the key
+    safeBuffer                      m_tagBuf;       // Holder of authentication tag
 	unsigned int					m_keyLen;
 	bool							m_initialised;	// Is the context ready to work?
 	unsigned char					m_lastBlock[MAX_BLOCK_SIZE];
