@@ -23,7 +23,7 @@
  * XSECCryptoSymmetricKey := Bulk encryption algorithms should all be
  *							implemented via this interface
  *
- * $Id: WinCAPICryptoSymmetricKey.cpp 1352680 2012-06-21 20:58:07Z scantor $
+ * $Id: WinCAPICryptoSymmetricKey.cpp 1655508 2015-01-29 02:05:56Z scantor $
  *
  */
 
@@ -525,7 +525,7 @@ unsigned int WinCAPICryptoSymmetricKey::encrypt(const unsigned char * inBuf,
 	   Otherwise we get some nasty errors due to rounding calculations when inputs
 	   are too small */
 
-	if (m_bytesInLastBlock + inLength < m_blockSize) {
+	if (inLength < m_blockSize) {
 		// Not enough input data
 		memcpy(&m_lastBlock[m_bytesInLastBlock], inBuf, inLength);
 		m_bytesInLastBlock += inLength;
