@@ -23,7 +23,7 @@
  * XSECEnv := Configuration class - used by the other classes to retrieve
  *            information on the environment they are working under
  *
- * $Id: XSECEnv.hpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XSECEnv.hpp 1350043 2012-06-13 22:31:04Z scantor $
  *
  */
 
@@ -158,6 +158,15 @@ public:
 
 	void setXENCNSPrefix(const XMLCh * prefix);
 
+	/**
+	 * \brief Set prefix for XENC 1.1 nodes
+	 *
+	 * Set the namespace prefix the library will use when creating
+	 * nodes in the XENC 1.1 namespace
+	 */
+
+	void setXENC11NSPrefix(const XMLCh * prefix);
+
 	/**
 	 * \brief Set prefix for XKMS nodes
 	 *
@@ -216,6 +225,16 @@ public:
 
 	const XMLCh * getXENCNSPrefix(void) const {return mp_xencPrefixNS;}
 
+	/**
+	 * \brief Get the NS Prefix being used for XENC 1.1 elements.
+	 *
+	 * @returns A pointer to the buffer holding the prefix
+	 * @see #setXENC11NSPrefix
+	 *
+	 */
+
+	const XMLCh * getXENC11NSPrefix() const {return mp_xenc11PrefixNS;}
+
 	/**
 	 * \brief Get namespace prefix for XKMS nodes
 	 *
@@ -524,11 +543,12 @@ private:
 	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument					
 								* mp_doc;
 	// For creating functions
-	XMLCh 						* mp_prefixNS;
-    XMLCh                       * mp_11PrefixNS;
+	XMLCh						* mp_prefixNS;
+	XMLCh						* mp_11PrefixNS;
 	XMLCh						* mp_ecPrefixNS;
 	XMLCh						* mp_xpfPrefixNS;
 	XMLCh						* mp_xencPrefixNS;
+	XMLCh						* mp_xenc11PrefixNS;
 	XMLCh						* mp_xkmsPrefixNS;
 
 	// Resolvers
