@@ -25,7 +25,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: txfmout.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: txfmout.cpp 1655934 2015-01-30 04:36:29Z scantor $
  *
  */
 
@@ -57,7 +57,6 @@
 
 #if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-# define _MAX_PATH PATH_MAX
 #else
 # if defined(HAVE_DIRECT_H)
 #  include <direct.h>
@@ -502,10 +501,14 @@ int main(int argc, char **argv) {
 		theResolver;
 		 
 	// Map out base path of the file
-	char path[_MAX_PATH];
-	char baseURI[(_MAX_PATH * 2) + 10];
-	getcwd(path, _MAX_PATH);
-
+#if XSEC_HAVE_GETCWD_DYN
+	char *path = getcwd(NULL, 0);
+	char *baseURI = (char*)malloc(strlen(path) + 8 + 1 + strlen(filename) + 1);
+#else
+	char path[PATH_MAX];
+	char baseURI[(PATH_MAX * 2) + 10];
+	getcwd(path, PATH_MAX);
+#endif
 	strcpy(baseURI, "file:///");
 	strcat(baseURI, path);
 	strcat(baseURI, "/");
@@ -526,7 +529,10 @@ int main(int argc, char **argv) {
 	baseURI[lastSlash + 1] = '\0';
 
 	theResolver.setBaseURI(MAKE_UNICODE_STRING(baseURI));
-
+#if XSEC_HAVE_GETCWD_DYN
+	free(path);
+	free(baseURI);
+#endif
 	sig->setURIResolver(&theResolver);
 
 
