@@ -22,7 +22,7 @@
  *
  * TXFMSB := Class that takes an input from a safeBuffer to start a pipe
  *
- * $Id: TXFMSB.hpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: TXFMSB.hpp 1350038 2012-06-13 22:21:29Z scantor $
  *
  */
 
@@ -53,8 +53,8 @@ public:
 	// Methods to set the inputs
 
 	virtual void setInput(TXFMBase *newInput);
-	void setInput(const safeBuffer sbIn);
-	void setInput(const safeBuffer sbIn, unsigned int sbSize); 
+	void setInput(const safeBuffer& sbIn);
+	void setInput(const safeBuffer& sbIn, unsigned int sbSize); 
 
 	// Methods to get tranform output type and input requirement
 
