@@ -22,7 +22,7 @@
  *
  * XSECDOMUtils:= Utilities to manipulate DOM within XML-SECURITY
  *
- * $Id: XSECDOMUtils.cpp 1139478 2011-06-25 01:39:11Z scantor $
+ * $Id: XSECDOMUtils.cpp 1350044 2012-06-13 22:31:37Z scantor $
  *
  */
 
@@ -88,7 +88,7 @@ const XMLCh * getXPFLocalName(const DOMNode * node) {
 
 }
 
-const XMLCh DSIG_EXPORT * getXENCLocalName(const DOMNode *node) {
+const XMLCh * getXENCLocalName(const DOMNode *node) {
 
 	// XML Encryption namespace node
 
@@ -99,7 +99,18 @@ const XMLCh DSIG_EXPORT * getXENCLocalName(const DOMNode *node) {
 
 }
 
-const XMLCh DSIG_EXPORT * getXKMSLocalName(const DOMNode *node) {
+const XMLCh * getXENC11LocalName(const DOMNode *node) {
+
+	// XML Encryption 1.1 namespace node
+
+	if (!strEquals(node->getNamespaceURI(), DSIGConstants::s_unicodeStrURIXENC11))
+		return NULL;
+	else
+		return node->getLocalName();
+
+}
+
+const XMLCh * getXKMSLocalName(const DOMNode *node) {
 
 	// XKMS namespace node
 
