@@ -28,7 +28,7 @@
  * NOTE: Much code taken from Xerces, and the cross platform interfacing is
  * no-where near as nice.
  *
- * $Id: XSECBinHTTPURIInputStream.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XSECBinHTTPURIInputStream.cpp 1655509 2015-01-29 02:15:59Z scantor $
  *
  */
 
@@ -107,18 +107,18 @@ void XSECBinHTTPURIInputStream::Initialize() {
 			ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
 		}
 		else {
-			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,_T("WSAStartup"));
-			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,_T("WSACleanup"));
-			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,_T("gethostbyname"));
-			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,_T("inet_addr"));
-			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,_T("gethostbyaddr"));
-			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,_T("htons"));
-			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,_T("socket"));
-			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,_T("connect"));
-			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,_T("send"));
-			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,_T("recv"));
-			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,_T("shutdown"));
-			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,_T("closesocket"));
+			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,"WSAStartup");
+			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,"WSACleanup");
+			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,"gethostbyname");
+			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,"inet_addr");
+			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,"gethostbyaddr");
+			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,"htons");
+			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,"socket");
+			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,"connect");
+			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,"send");
+			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,"recv");
+			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,"shutdown");
+			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,"closesocket");
 
 			if(startup == NULL ||
 				gWSACleanup == NULL ||
