@@ -23,7 +23,7 @@
  * XSECAlgorithmHandlerDefault := Interface class to define handling of
  *								  default encryption algorithms
  *
- * $Id: XENCAlgorithmHandlerDefault.hpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XENCAlgorithmHandlerDefault.hpp 1350045 2012-06-13 22:33:10Z scantor $
  *
  */
 
@@ -116,7 +116,9 @@ private:
 		XSECCryptoKey * key,
 		XSECCryptoKey::KeyType &kt,
 		XSECCryptoSymmetricKey::SymmetricKeyType &skt,
-		bool &isSymmetricKeyWrap);
+		bool &isSymmetricKeyWrap,
+        XSECCryptoSymmetricKey::SymmetricKeyMode &skm,
+        unsigned int& taglen);
 	unsigned int doRSADecryptToSafeBuffer(
 		TXFMChain * cipherText,
 		XENCEncryptionMethod * encryptionMethod,
@@ -129,6 +131,11 @@ private:
 		XSECCryptoKey * key,
 		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc,
 		safeBuffer & result);
+	unsigned int doGCMDecryptToSafeBuffer(
+		TXFMChain * cipherText,
+		XSECCryptoKey * key,
+        unsigned int taglen,
+		safeBuffer & result);
 	unsigned int unwrapKeyAES(
    		TXFMChain * cipherText,
 		XSECCryptoKey * key,
@@ -145,7 +152,6 @@ private:
    		TXFMChain * cipherText,
 		XSECCryptoKey * key,
 		safeBuffer & result);
-
 };
 
 /*\@}*/
