@@ -23,7 +23,7 @@
  * XENCEncryptedTypeImpl := Implementation of the EncryptedType interface
  * element
  *
- * $Id: XENCEncryptedTypeImpl.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XENCEncryptedTypeImpl.cpp 1350038 2012-06-13 22:21:29Z scantor $
  *
  */
 
@@ -333,7 +333,8 @@ TXFMChain * XENCEncryptedTypeImpl::createCipherTXFMChain(void) {
             TXFMSB *sb;
             XSECnew(sb, TXFMSB(mp_env->getParentDocument()));
 
-            sb->setInput(safeBuffer(b64.get()));
+            safeBuffer temp(b64.get());
+            sb->setInput(temp);
 
             // Create a chain
             XSECnew(chain, TXFMChain(sb));
